DECLARE SUB RunLevel ()
DECLARE FUNCTION ConfirmQuit% ()
DECLARE SUB NewLevel ()
DECLARE SUB DisplayHelp (isQuickHelp%)
DECLARE SUB CenterText (row%, text$)
DECLARE SUB DrawLevelSpriteMap ()
DECLARE SUB DisplayTileInfo ()

DEFINT A-Z
'$DYNAMIC
'$INCLUDE: 'XMASCOM.BI'

DECLARE SUB RemoveSpriteAtCursor ()
DECLARE SUB EraseSprite (x%, y%)
DECLARE SUB DrawCursor (cursorColor%)
DECLARE SUB DrawPreview ()
DECLARE SUB DrawBackground (drawGrid%, redrawSprites%)
DECLARE SUB DrawSprite (xOffset%, yOffset%, SpriteId%)
DECLARE SUB SelectDrawSprite ()
DECLARE SUB PlaceSpriteAtCursor ()
DECLARE SUB MoveCursor (xDelta%, yDelta%)
DECLARE SUB DrawBgGrid ()
DECLARE FUNCTION GetCursorModeStr$ ()
DECLARE SUB SaveLevel ()
DECLARE SUB LoadLevel ()
DECLARE SUB EditorExitScreen ()
DECLARE SUB ClearKbBuffer ()
DECLARE FUNCTION CheckFileExists% (filename$)
DECLARE SUB LoadSpriteIdFile ()
DECLARE SUB LoadSpriteIntoMap (SpriteId%, filename$, SpriteMap%())


TYPE CursorType
    x AS INTEGER
    y AS INTEGER
    SpriteId AS INTEGER
    mode AS INTEGER
END TYPE

TYPE TileInfo
    typeId AS INTEGER
    value AS INTEGER
    attributes AS STRING * 4
END TYPE

'Modes for cursor.
CONST LEVELMODE = 1
CONST ENEMYMODE = 2
CONST ITEMMODE = 3
CONST MARKERMODE = 4  'for start/end locations
CONST PREVIEWMODE = 5



DIM SHARED SpriteMap(MAXSPRITES, MAXSPRITEWIDTH, MAXSPRITEHEIGHT) AS INTEGER
DIM SHARED SpriteIdFileNameMap(MAXSPRITES) AS STRING * 12
DIM SHARED LevelSpriteMap(MAXHORIZONTALSPRITES, MAXVERTICALSPRITES) AS INTEGER
DIM SHARED LevelTileInfoMap(MAXHORIZONTALSPRITES, MAXVERTICALSPRITES) AS TileInfo

DIM SHARED Cursor AS CursorType

DIM SHARED LevelFileName AS STRING * 12

DIM SHARED prevSpriteCursorX AS INTEGER
DIM SHARED prevSpriteCursorY AS INTEGER

DIM SHARED IsLevelSaved AS INTEGER

Cursor.x = 1
Cursor.y = 1
Cursor.SpriteId = 1
Cursor.mode = LEVELMODE

prevSpriteCursorX = 1
prevSpriteCursorY = 1


IsLevelSaved = FALSE

CALL LoadSpriteIdFile


SCREEN 13
CLS

CALL DrawBackground(TRUE, FALSE)
CALL DrawCursor(14)

quit% = FALSE

WHILE quit% = FALSE

    start! = TIMER

    in% = INP(96)
    LOCATE 3, 32
    PRINT "KEY:"; in%


    cursorXDelta% = 0
    cursorYDelta% = 0

    SELECT CASE in%

        CASE TABKEY
            Cursor.mode = Cursor.mode + 1
            IF Cursor.mode > MARKERMODE THEN Cursor.mode = 1
            CALL DrawBackground(FALSE, FALSE)


        CASE F1KEY
            CALL DisplayHelp(FALSE)

        CASE F2KEY
            CALL SaveLevel

        CASE F3KEY
            CALL LoadLevel

        CASE F5KEY
            CALL RunLevel
       
        CASE F6KEY
            CALL DrawPreview

        CASE F8KEY
            CALL DisplayHelp(TRUE)

        CASE DELKEY
            CALL RemoveSpriteAtCursor

        CASE UPARROW
            cursorYDelta% = -1
       
        CASE DOWNARROW
            cursorYDelta% = 1

        CASE LEFTARROW
            cursorXDelta% = -1

        CASE RIGHTARROW
            cursorXDelta% = 1

        CASE SPACEKEY
            CALL PlaceSpriteAtCursor

        CASE NKEY
            CALL NewLevel

        CASE SKEY
            CALL SelectDrawSprite

        CASE QKEY
            quit% = ConfirmQuit%

    END SELECT

    CALL ClearKbBuffer

    IF cursorYDelta% <> 0 OR cursorXDelta% <> 0 THEN
        CALL MoveCursor(cursorXDelta%, cursorYDelta%)
        CALL DisplayTileInfo
    END IF


    'fps control
    WHILE TIMER - start! <= .07: WEND

WEND

CALL EditorExitScreen


END

REM $STATIC
FUNCTION ConfirmQuit%

    CALL ClearKbBuffer

    LINE (20, 35)-(250, 50), 2, BF
    LINE (20, 50)-(250, 75), 7, BF
    COLOR 15
    LOCATE 6, 4
    PRINT "QUIT EDITOR"
    LOCATE 8, 4
    INPUT "Are you sure? [Y/N] ", confirm$

    IF UCASE$(LTRIM$(RTRIM$(confirm$))) <> "Y" THEN
        CLS
        CALL DrawBackground(TRUE, TRUE)
        CALL DrawCursor(14)

        ConfirmQuit% = FALSE

        EXIT FUNCTION
    END IF

    ConfirmQuit% = TRUE

END FUNCTION

SUB DisplayHelp (isQuickHelp%)


    IF isQuickHelp% THEN
        GOTO QuickHelp
    END IF

    CLS
    COLOR 10

    in% = 0

    CALL CenterText(1, "HELP")
    CALL CenterText(2, "______")
 
    LOCATE 3
    COLOR 15
    PRINT "MAIN SCREEN CONTROLS"
    COLOR 7
    PRINT
    PRINT "        F1 - This help screen"
    PRINT "        F2 - Save Level"
    PRINT "        F3 - Load Level"
    PRINT "        F5 - Display preview level"
    PRINT "        F8 - Quick Help"
    PRINT "ARROW KEYS - Move cursor around"
    PRINT "       DEL - Delete sprite at cursor"
    PRINT "     SPACE - Place sprite at cursor"
    PRINT "       TAB - Cycle sprite types"
    PRINT "         N - New Level"
    PRINT "         Q - Quit Program"
    PRINT "         S - Select sprite"
    PRINT
    COLOR 15
    PRINT "SPRITE SELECTION CONTROLS"
    PRINT
    COLOR 7
    PRINT "ARROW KEYS - Move cursor around"
    PRINT "       ESC - Return without selecting"
    PRINT "     ENTER - Select sprite at cursor"
    

    GOSUB CheckInput

    CLS
    COLOR 15
    PRINT "LOADING A LEVEL"
    PRINT
    COLOR 7
    PRINT "When loading a level, the current"
    PRINT "directory is assumed. You can use a"
    PRINT "relative path or a absolute path if"
    PRINT "needed in the load level prompt."
    PRINT
    PRINT "The file extension '.LVL' is assumed if"
    PRINT "no file extension is given."
    PRINT
    PRINT
    COLOR 15
    PRINT "SAVING A LEVEL"
    PRINT
    COLOR 7
    PRINT "The same holds true when saving a"
    PRINT "level as loading a level. If only a "
    PRINT "file name is given, it will be saved"
    PRINT "to the current directory with the"
    PRINT "extension '.LVL'"

    GOSUB CheckInput

    CLS
    COLOR 15
    PRINT "SPRITE TYPES"
    PRINT
    COLOR 7
    PRINT "For all sprite types besides LEVEL"
    PRINT "additional attributes are required to"
    PRINT "be set"
    PRINT
    PRINT "   LEVEL - Non-interactive tiles that"
    PRINT "           the player can walk on."
    PRINT
    PRINT "   ENEMY - Enemies that attack the"
    PRINT "           player."
    PRINT
    PRINT "    ITEM - Item that the player can"
    PRINT "           pick up"
    PRINT
    PRINT "  MARKER - Special tile that marks a"
    PRINT "           location or an interactive"
    PRINT "           tile."
    PRINT

    GOSUB CheckInput


QuickHelp:

    CLS
    COLOR 10
    PRINT "ENEMY ATTRIBUTES"
    PRINT
    COLOR 7
    PRINT "For all enemies, the ATTRIBUTE value"
    PRINT "sets their 'sleep' time after"
    PRINT "colliding with the player. The VALUE"
    PRINT "sets the SPRITEID of the first"
    PRINT "animation sprite. This should be left"
    PRINT "blank as it will be automatically"
    PRINT "populated."
    PRINT
    COLOR 15
    PRINT "ATTRIBUTE POSTFIX (ex: 100S, 150R)"
    PRINT
    COLOR 7
    PRINT "    NONE or S - Snooper"
    PRINT "            R - Runner"
    PRINT "            B - Burgler"
    PRINT
    PRINT "NOTE: Sleep time is required to be"
    PRINT "      three characters so values less"
    PRINT "      than three characters shoudl have"
    PRINT "      leading zeros. Ex: 005S, 050R, Etc"
    
    GOSUB CheckInput


    CLS
    COLOR 10
    PRINT "ITEM ATTRIBUTES"
    PRINT
    COLOR 15
    PRINT "ALARM CLOCK"
    COLOR 7
    PRINT "  ATTRIBUTE: WAKE"
    PRINT "      VALUE: Number to add to all"
    PRINT "             enemies step speed"
    PRINT
    COLOR 15
    PRINT "COOKIES"
    COLOR 7
    PRINT "  ATTRIBUTE: COOK"
    PRINT "      VALUE: If blank, +50 Spirit. Other"
    PRINT "             value is +VALUE"
    PRINT
    COLOR 15
    PRINT "HELPER ELF"
    COLOR 7
    PRINT "  ATTRIBUTE: HELF"
    PRINT "      VALUE: Ignored. Always one present"
    PRINT
    COLOR 15
    PRINT "HOUR GLASS"
    COLOR 7
    PRINT "  ATTRIBUTE: TIME"
    PRINT "      VALUE: Number of timer ticks to"
    PRINT "             add to current level timer."
   
    
    GOSUB CheckInput
    CLS
    COLOR 15
    PRINT "KEY"
    COLOR 7
    PRINT "  ATTRIBUTE: KEY"
    PRINT "      VALUE: Id of door(s) to unlock."
    PRINT "             These are set in the door"
    PRINT "             MARKER tile value."
    PRINT
    COLOR 15
    PRINT "MILK"
    COLOR 7
    PRINT "  ATTRIBUTE: MILK"
    PRINT "      VALUE: If blank, +100 Sprite."
    PRINT "             Other value is +VALUE"
    PRINT
    COLOR 15
    PRINT "ONE UP"
    COLOR 7
    PRINT "  ATTRIBUTE: LIFE"
    PRINT "      VALUE: Number of lives to add"
    PRINT
    COLOR 15
    PRINT "Z's"
    COLOR 7
    PRINT "  ATTRIBUTE: SLEP"
    PRINT "      VALUE: Number of game ticks to"
    PRINT "             sleep all enemies."


    GOSUB CheckInput

    CLS
    COLOR 10
    PRINT "MARKER ATTRIBUTES"
    PRINT
    COLOR 15
    PRINT "DOOR MARKER"
    COLOR 7
    PRINT "  ATTRIBUTE: DOOR"
    PRINT "      VALUE: Id of key that unlocks"
    PRINT "             it."
    PRINT
    COLOR 15
    PRINT "LAVA MARKER"
    COLOR 7
    PRINT "  ATTRIBUTE: LAVA"
    PRINT "      VALUE: Not used. Marks tile as"
    PRINT "             foreground and passible."
    PRINT
    COLOR 15
    PRINT "PLAYER MARKER (Player start position)"
    COLOR 7
    PRINT "  ATTRIBUTE: PLAY"
    PRINT "      VALUE: First SPRITEID of player"
    PRINT "             sprite animation. If blank"
    PRINT "             will be current SPRITEID"
    PRINT
    COLOR 15
    PRINT "SLED MARKER"
    COLOR 7
    PRINT "  ATTRIBUTE: SLED"
    PRINT "      VALUE: Number of presents to"
    PRINT "             deliver in the level"

    GOSUB CheckInput


    CLS
    COLOR 15
    PRINT "SPRING MARKER"
    COLOR 7
    PRINT "  ATTRIBUTE: JUMP"
    PRINT "      VALUE: Y value to jump player to"
    PRINT "             1 = top of screen."
    PRINT
    COLOR 15
    PRINT "TELEPORT END MARKER"
    COLOR 7
    PRINT "  ATTRIBUTE: TELE"
    PRINT "      VALUE: ID (Value) of teleport"
    PRINT "             start (TELS) to send player"
    PRINT "             to."
    PRINT
    COLOR 15
    PRINT "TELEPORT START MARKER"
    COLOR 7
    PRINT "  ATTRIBUTE: TELS"
    PRINT "      VALUE: ID (Value) of teleport"
    PRINT "             end (TELE) to send player"
    PRINT "             to."
    PRINT
    COLOR 15
    PRINT "TREE MARKER"
    COLOR 7
    PRINT "  ATTRIBUTE: TREE"
    PRINT "      VALUE: Number of game ticks to"
    PRINT "             set the level timer to."


    GOSUB CheckInput

    CLS
    COLOR 15
    PRINT "WATER MARKER"
    COLOR 7
    PRINT "  ATTRIBUTE: WATR"
    PRINT "      VALUE: Not used. Marks tile as"
    PRINT "             foreground and passible."
    PRINT
    COLOR 14
    PRINT "BONUS LEVEL SPECIFIC"
    PRINT
    COLOR 15
    PRINT "BONUS LEVEL ITEM"
    COLOR 7
    PRINT "  ATTRIBUTE: XTRA"
    PRINT "      VALUE: 0 - Reindeer bonus level"
    PRINT "             1 - Endless presents lvl"
    PRINT
    COLOR 15
    PRINT "BONUS LEVEL MARKER (Use blank sprite)"
    COLOR 7
    PRINT "  ATTRIBUTE: XTRA"
    PRINT "      VALUE: 0 - Reindeer bonus level"
    PRINT "             1 - Endless presents lvl"
    PRINT
    COLOR 15
    PRINT "REINDEER ITEM"
    COLOR 7
    PRINT "  ATTRIBUTE: DEER"
    PRINT "      VALUE: X tile number to place"
    PRINT "             picked up deer on first"
    PRINT "             Y tile line."



    GOSUB CheckInput

    CLS
    CALL DrawBackground(TRUE, TRUE)
    CALL DrawCursor(14)

    EXIT SUB


CheckInput:

    COLOR 11
    CALL CenterText(23, "Enter - Next Page   ESC - Return")
   
    DO
        CALL ClearKbBuffer
        in% = INP(96)
    LOOP WHILE in% <> ENTERKEY AND in% <> ESCKEY


    IF in% = ESCKEY THEN
        CLS
        CALL DrawBackground(TRUE, TRUE)
        CALL DrawCursor(14)
        EXIT SUB
    END IF

    RETURN


END SUB

SUB DisplayTileInfo
   
    levelX% = Cursor.x / MAXSPRITEWIDTH + 1
    levelY% = Cursor.y / MAXSPRITEHEIGHT + 1
    COLOR 15
    LOCATE 11, 32
    PRINT "TILE INFO"
    LOCATE 12, 32
    PRINT "ID:"; LevelTileInfoMap(levelX%, levelY%).typeId
    LOCATE 13, 32
    PRINT SPACE$(8)
    LOCATE 13, 32
    PRINT "A: "; LevelTileInfoMap(levelX%, levelY%).attributes
    LOCATE 14, 32
    PRINT SPACE$(8)
    LOCATE 14, 32
    PRINT "V:"; LevelTileInfoMap(levelX%, levelY%).value
   

END SUB

SUB DrawBackground (drawGrid%, redrawSprites%)

    IF redrawSprites% THEN
        CALL DrawLevelSpriteMap
    END IF
  
    IF drawGrid% THEN
        CALL DrawBgGrid
    END IF


    COLOR 15
    LOCATE 22
    PRINT "Mode: "; GetCursorModeStr$

    LOCATE 22, 15
    PRINT "File: "; LevelFileName


    LOCATE 5, 32
    PRINT "SPRITE"
    LOCATE 9, 32
    PRINT "("; Cursor.SpriteId; ")"
    CALL DrawSprite(265, 42, Cursor.SpriteId)

END SUB

SUB DrawBgGrid

    FOR yOffset% = 1 TO MAXLEVELHEIGHT STEP MAXSPRITEHEIGHT
        FOR xOffset% = 1 TO MAXLEVELWIDTH STEP MAXSPRITEWIDTH
   
            LINE (xOffset%, yOffset%)-(xOffset% + MAXSPRITEWIDTH, yOffset% + MAXSPRITEHEIGHT), 8, B

        NEXT xOffset%
    NEXT yOffset%



END SUB

SUB DrawCursor (cursorColor%)

    LINE (Cursor.x, Cursor.y)-(Cursor.x + MAXSPRITEWIDTH, Cursor.y + MAXSPRITEHEIGHT), cursorColor%, B

END SUB

SUB DrawLevelSpriteMap

    FOR mapY% = 1 TO UBOUND(LevelSpriteMap, 2)
        FOR mapX% = 1 TO UBOUND(LevelSpriteMap, 1)

            'need the upper left most corner + 1
            spriteDrawX% = mapX% * MAXSPRITEWIDTH - MAXSPRITEWIDTH
            spriteDrawY% = mapY% * MAXSPRITEHEIGHT - MAXSPRITEHEIGHT

            CALL DrawSprite(spriteDrawX%, spriteDrawY%, LevelSpriteMap(mapX%, mapY%))

        NEXT mapX%
    NEXT mapY%

END SUB

SUB DrawPreview

    CLS

    prevMode% = Cursor.mode
    Cursor.mode = PREVIEWMODE

    CALL DrawBackground(FALSE, TRUE)
    LOCATE 23
    COLOR 7
    PRINT "Press any key to return to editor"

    CALL ClearKbBuffer

    DO: LOOP WHILE INKEY$ = ""

    LOCATE 23
    PRINT SPACE$(33)

    Cursor.mode = prevMode%
   
    CALL DrawBackground(TRUE, FALSE)
   
END SUB

SUB DrawSprite (xOffset%, yOffset%, SpriteId%)

    FOR y% = 1 TO MAXSPRITEHEIGHT
        FOR x% = 1 TO MAXSPRITEWIDTH

            'skips 'PSET'ing on blank sprite data to save draw time.

            IF SpriteMap(SpriteId%, x%, y%) > 0 THEN

                ' Much faster than PSET directly!
                DEF SEG = &HA000
                    POKE (x% + xOffset%) + ((y% + yOffset%) * 320&), SpriteMap(SpriteId%, x%, y%)
                DEF SEG


                'PSET (x% + xOffset%, y% + yOffset%), SpriteMap(SpriteId%, x%, y%)
            END IF

        NEXT x%
    NEXT y%

END SUB

SUB EraseSprite (x%, y%)

    'fill grid block with black then draw grid box

    LINE (x%, y%)-(x% + MAXSPRITEWIDTH, y% + MAXSPRITEHEIGHT), 0, BF
    LINE (x%, y%)-(x% + MAXSPRITEWIDTH, y% + MAXSPRITEHEIGHT), 8, B

END SUB

FUNCTION GetCursorModeStr$

    modeStr$ = "UNKNOWN"
   
    SELECT CASE Cursor.mode

        CASE LEVELMODE
            modeStr$ = "LEVEL  "

        CASE ENEMYMODE
            modeStr$ = "ENEMY  "

        CASE ITEMMODE
            modeStr$ = "ITEM   "

        CASE MARKERMODE
            modeStr$ = "MARK   "

        CASE PREVIEWMODE
            modeStr$ = "PREVIEW"

    END SELECT


    GetCursorModeStr$ = modeStr$

END FUNCTION

SUB LoadLevel

    LINE (20, 35)-(250, 50), 2, BF
    LINE (20, 50)-(250, 75), 7, BF
    COLOR 15
    LOCATE 6, 4
    PRINT "LOAD LEVEL"
    LOCATE 8, 4
    INPUT "FILENAME: ", filename$

    IF LTRIM$(RTRIM$(filename$)) = "" THEN
        CLS
        CALL DrawBackground(TRUE, TRUE)
        CALL DrawCursor(14)

        EXIT SUB
    END IF

    filename$ = UCASE$(LTRIM$(RTRIM$(filename$)))
    finalFileName$ = LEFT$(filename$, 8)

    dotIdx% = INSTR(filename$, ".")
   
    IF dotIdx% = FALSE THEN
         finalFileName$ = finalFileName$ + ".LVL"
    ELSE
        extension$ = RIGHT$(filename$, LEN(filename$) - dotIdx%)
        finalFileName$ = finalFileName$ + "." + LEFT$(extension$, 3)
    END IF

    
    'fail to load if file doesn't exist.
    IF CheckFileExists%(finalFileName$) = FALSE THEN
        CLS
        CALL DrawBackground(TRUE, TRUE)
        CALL DrawCursor(14)
        EXIT SUB
    END IF

    OPEN finalFileName$ FOR INPUT AS #1
        FOR y% = 1 TO UBOUND(LevelSpriteMap, 2)
            FOR x% = 1 TO UBOUND(LevelSpriteMap, 1)
                INPUT #1, LevelSpriteMap(x%, y%)
            NEXT x%
        NEXT y%
        FOR y% = 1 TO UBOUND(LevelTileInfoMap, 2)
            FOR x% = 1 TO UBOUND(LevelTileInfoMap, 1)
                INPUT #1, LevelTileInfoMap(x%, y%).typeId, LevelTileInfoMap(x%, y%).value, LevelTileInfoMap(x%, y%).attributes
            NEXT x%
        NEXT y%
    CLOSE #1
     
    LevelFileName = finalFileName$

    CLS
    CALL DrawBackground(TRUE, TRUE)
    CALL DrawCursor(14)

    LOCATE 23, 1
    PRINT "Loaded level: "; finalFileName$
   
   
    IsLevelSaved = FALSE

END SUB

SUB LoadSpriteIdFile

    fileExists% = CheckFileExists%(SPRITEIDFILE)

    IF fileExists% THEN

        OPEN SPRITEIDFILE FOR INPUT AS #1

            WHILE NOT EOF(1)
     
                INPUT #1, tempId%, tempFileName$
 
                IF tempId% < MAXSPRITES THEN
                    SpriteIdFileNameMap(tempId%) = tempFileName$

                    IF LTRIM$(RTRIM$(tempFileName$)) <> "" THEN
                        CALL LoadSpriteIntoMap(tempId%, tempFileName$, SpriteMap())
                    END IF

                END IF
            WEND
        CLOSE #1

        LOCATE 22, 2
        COLOR 15
        PRINT "Loaded sprite id file: "; SPRITEIDFILE

    ELSE
        LOCATE 22, 2
        COLOR 15
        PRINT SPRITEIDFILE; " Doesn't exist to load!"

    END IF


END SUB

SUB MoveCursor (xDelta%, yDelta%)

    IF xDelta% = 0 AND yDelta% = 0 THEN
        EXIT SUB
    END IF


    IF xDelta% > 0 THEN
        xDelta% = MAXSPRITEWIDTH
    ELSEIF xDelta% < 0 THEN
        xDelta% = -MAXSPRITEWIDTH
    END IF


    IF yDelta% > 0 THEN
        yDelta% = MAXSPRITEHEIGHT
    ELSEIF yDelta% < 0 THEN
        yDelta% = -MAXSPRITEHEIGHT
    END IF

    'draw cursor mask at old location.
    CALL DrawCursor(8)

    startX% = Cursor.x
    startY% = Cursor.y

    Cursor.x = Cursor.x + xDelta%
    Cursor.y = Cursor.y + yDelta%

    IF Cursor.x > MAXLEVELWIDTH OR Cursor.y > MAXLEVELHEIGHT OR Cursor.x < 1 OR Cursor.y < 1 THEN
        Cursor.x = startX%
        Cursor.y = startY%
    END IF

    CALL DrawCursor(14)

END SUB

SUB NewLevel

    CALL ClearKbBuffer

    'IF IsLevelSaved = FALSE THEN

    '    LINE (20, 35)-(250, 50), 2, BF
    '    LINE (20, 50)-(250, 75), 7, BF
    '    COLOR 15
    '    LOCATE 6, 4
    '    PRINT "NEW LEVEL"
    '    LOCATE 8, 4
    '    PRINT "Current level not saved."
    '    LOCATE 9, 4
    '    INPUT "Discard changes? [Y/N]", discard$
    '
    '    IF UCASE$(LTRIM$(RTRIM$(discard$))) <> "Y" THEN
    '
    '        CLS
    '        CALL DrawBackground(TRUE, TRUE)
    '        CALL DrawCursor(14)
    '
    '        EXIT SUB
    '    END IF
    'END IF



    

    LINE (20, 35)-(250, 50), 2, BF
    LINE (20, 50)-(250, 75), 7, BF
    COLOR 15
    LOCATE 6, 4
    PRINT "NEW LEVEL"
    LOCATE 8, 4
    INPUT "Are you sure? [Y/N] ", confirm$

    IF UCASE$(LTRIM$(RTRIM$(confirm$))) <> "Y" THEN
        CLS
        CALL DrawBackground(TRUE, TRUE)
        CALL DrawCursor(14)

        EXIT SUB
    END IF
    
    
        
    REDIM LevelSpriteMap(MAXHORIZONTALSPRITES, MAXVERTICALSPRITES) AS INTEGER

    REDIM LevelTileInfoMap(MAXHORIZONTALSPRITES, MAXVERTICALSPRITES) AS TileInfo
        

    LevelFileName = "NEWBOARD.LVL"

    CLS
    CALL DrawBackground(TRUE, TRUE)
    CALL DrawCursor(14)

    LOCATE 23, 1
    PRINT "Started New Level"

    IsLevelSaved = FALSE

END SUB

SUB PlaceSpriteAtCursor

    IF Cursor.SpriteId < 1 OR Cursor.SpriteId > MAXSPRITES THEN
        EXIT SUB
    END IF

    CALL EraseSprite(Cursor.x, Cursor.y)
    CALL DrawSprite(Cursor.x, Cursor.y, Cursor.SpriteId)

    levelX% = Cursor.x / MAXSPRITEWIDTH + 1
    levelY% = Cursor.y / MAXSPRITEHEIGHT + 1
    
    LevelSpriteMap(levelX%, levelY%) = Cursor.SpriteId

    LevelTileInfoMap(levelX%, levelY%).typeId = Cursor.mode

    IF Cursor.mode <> LEVELMODE THEN

        CALL ClearKbBuffer

        LINE (20, 35)-(200, 50), 2, BF
        LINE (20, 50)-(200, 75), 7, BF
        COLOR 15
        LOCATE 6, 4
        PRINT "NEW SPRITE ATTRIBUTES"

        LOCATE 8, 4
        INPUT "Attribute: ", attrVal$
        LevelTileInfoMap(levelX%, levelY%).attributes = UCASE$(attrVal$)

        LOCATE 9, 4
        INPUT "Value: ", newVal$
        LevelTileInfoMap(levelX%, levelY%).value = VAL(newVal$)
      
        CLS
        CALL DrawBackground(TRUE, TRUE)
    ELSE
        LevelTileInfoMap(levelX%, levelY%).value = 0
        LevelTileInfoMap(levelX%, levelY%).attributes = BLOCKINGATTR
    END IF
     
    CALL DrawCursor(14)
    CALL DisplayTileInfo

    COLOR 7
    LOCATE 23
    PRINT "Placed: "; levelX%; ","; levelY%

END SUB

SUB RemoveSpriteAtCursor
   
    CALL EraseSprite(Cursor.x, Cursor.y)
   
    levelX% = Cursor.x / MAXSPRITEWIDTH + 1
    levelY% = Cursor.y / MAXSPRITEHEIGHT + 1
   
    LevelSpriteMap(levelX%, levelY%) = 0
    LevelTileInfoMap(levelX%, levelY%).typeId = TILETYPEUNSET
    LevelTileInfoMap(levelX%, levelY%).value = 0
    LevelTileInfoMap(levelX%, levelY%).attributes = ""

    CALL DrawCursor(14)
    CALL DisplayTileInfo

    COLOR 7
    LOCATE 23
    PRINT SPACE$(40)
    LOCATE 23
    PRINT "Removed: "; levelX%; ","; levelY%



END SUB

SUB RunLevel

    CALL ClearKbBuffer

    IF IsLevelSaved = FALSE OR LTRIM$(RTRIM$(LevelFileName)) = "" THEN

        LINE (20, 35)-(250, 50), 2, BF
        LINE (20, 50)-(250, 105), 7, BF
        COLOR 15
        LOCATE 6, 4
        PRINT "RUN LEVEL"
        LOCATE 8, 4
        PRINT "Current level not saved."
        LOCATE 9, 4
        PRINT "Please save level to run."
        LOCATE 11, 4
        PRINT "Press any key to return."
    
        DO: LOOP WHILE INKEY$ = ""
    
        CLS
        CALL DrawBackground(TRUE, TRUE)
        CALL DrawCursor(14)
    
        EXIT SUB
        
    END IF


    CLS
    
    'SHELL "qbx /ah /run sdrush /cmd " + LevelFileName + " /Q"
    
    SHELL ".\SDRUSH.EXE " + LevelFileName + " /Q"

    CLS
    CALL DrawBackground(TRUE, TRUE)
    CALL DrawCursor(14)


END SUB

SUB SaveLevel

    LINE (20, 35)-(250, 50), 2, BF
    LINE (20, 50)-(250, 75), 7, BF
    COLOR 15
    LOCATE 6, 4
    PRINT "SAVE LEVEL"
    LOCATE 8, 4
    INPUT "FILENAME: ", filename$

    IF LTRIM$(RTRIM$(filename$)) = "" THEN
        CLS
        CALL DrawBackground(TRUE, TRUE)
        CALL DrawCursor(14)

        EXIT SUB
    END IF

    filename$ = UCASE$(LTRIM$(RTRIM$(filename$)))
    finalFileName$ = LEFT$(filename$, 8)

    dotIdx% = INSTR(filename$, ".")
    
    IF dotIdx% = FALSE THEN
         finalFileName$ = finalFileName$ + ".LVL"
    ELSE
        extension$ = RIGHT$(filename$, LEN(filename$) - dotIdx%)
        finalFileName$ = finalFileName$ + "." + LEFT$(extension$, 3)
    END IF

    OPEN finalFileName$ FOR OUTPUT AS #1
        FOR y% = 1 TO UBOUND(LevelSpriteMap, 2)
            FOR x% = 1 TO UBOUND(LevelSpriteMap, 1)
                WRITE #1, LevelSpriteMap(x%, y%)
            NEXT x%
        NEXT y%
        FOR y% = 1 TO UBOUND(LevelTileInfoMap, 2)
            FOR x% = 1 TO UBOUND(LevelTileInfoMap, 1)

               
                'For player markers and enemy types , if value is not set,
                'use sprite id. Sprite id in value is used during
                'loading in the engine to set start of animation frames

                IF LevelTileInfoMap(x%, y%).typeId = TILETYPEMARKER AND LevelTileInfoMap(x%, y%).value = 0 AND LevelTileInfoMap(x%, y%).attributes = PLAYERATTR THEN
                     LevelTileInfoMap(x%, y%).value = LevelSpriteMap(x%, y%)
               
                ELSEIF LevelTileInfoMap(x%, y%).typeId = TILETYPEENEMY AND LevelTileInfoMap(x%, y%).value = 0 THEN
                     LevelTileInfoMap(x%, y%).value = LevelSpriteMap(x%, y%)
                END IF


                WRITE #1, LevelTileInfoMap(x%, y%).typeId, LevelTileInfoMap(x%, y%).value, LevelTileInfoMap(x%, y%).attributes
            NEXT x%
        NEXT y%
    CLOSE #1
      
    LevelFileName = finalFileName$

    CLS
    CALL DrawBackground(TRUE, TRUE)
    CALL DrawCursor(14)

    LOCATE 23, 1
    PRINT "Saved level: "; finalFileName$

    IsLevelSaved = TRUE


END SUB

SUB SelectDrawSprite

    CLS

    sid% = 1

    CALL DrawBgGrid

    FOR yOffset% = 1 TO MAXLEVELHEIGHT STEP MAXSPRITEHEIGHT
        FOR xOffset% = 1 TO MAXLEVELWIDTH STEP MAXSPRITEWIDTH
  
            IF sid% >= MAXSPRITES THEN EXIT FOR

            CALL DrawSprite(xOffset%, yOffset%, sid%)

            sid% = sid% + 1

        NEXT xOffset%
    NEXT yOffset%

    'save map cursor positions and reload for sprite selection.
    cursorPrevX% = Cursor.x
    cursorPrevY% = Cursor.y
    Cursor.x = prevSpriteCursorX
    Cursor.y = prevSpriteCursorY

    CALL DrawCursor(14)

    CALL ClearKbBuffer

    in% = 0
    quit% = FALSE

    WHILE quit% = FALSE

        start! = TIMER

        in% = INP(96)

        cursorXDelta% = 0
        cursorYDelta% = 0

        SELECT CASE in%

            CASE ESCKEY
                quit% = TRUE

            CASE ENTERKEY
                mapX% = Cursor.x / MAXSPRITEWIDTH + 1
                mapY% = Cursor.y / MAXSPRITEHEIGHT + 1

                selectedId% = MAXHORIZONTALSPRITES * (mapY% - 1) + mapX%

                IF selectedId% <= MAXSPRITES AND selectedId% > 0 THEN
                    Cursor.SpriteId = selectedId%
                END IF

                quit% = TRUE

            CASE UPARROW
                cursorYDelta% = -1
      
            CASE DOWNARROW
                cursorYDelta% = 1

            CASE LEFTARROW
                cursorXDelta% = -1

            CASE RIGHTARROW
                cursorXDelta% = 1

        END SELECT

        CALL MoveCursor(cursorXDelta%, cursorYDelta%)
        CALL ClearKbBuffer
         
        'fps control
        WHILE TIMER - start! <= .07: WEND


    WEND
   


    'save current and set back prev values before returning.
    prevSpriteCursorX = Cursor.x
    prevSpriteCursorY = Cursor.y
    Cursor.x = cursorPrevX%
    Cursor.y = cursorPrevY%

    CLS
    CALL DrawBackground(TRUE, TRUE)
    CALL DrawCursor(14)

END SUB

