DEFINT A-Z
'$DYNAMIC
'$INCLUDE: 'XMASCOM.BI'


DECLARE FUNCTION SaveSprite$ (existingSpriteFileName$)
DECLARE SUB ReverseSprite ()
DECLARE SUB EditorExitScreen ()
DECLARE SUB FillBackground ()
DECLARE SUB NewSprite (interactive%)
DECLARE FUNCTION CheckFileExists% (fileName$)
DECLARE FUNCTION LoadSprite$ (existingSpriteFileName$)
DECLARE SUB ReturnToEditor (textX%, textY%, textColor%)
DECLARE SUB SetQuickCursorColor (newColor%)
DECLARE SUB PreviewSprite ()
DECLARE SUB SelectDrawColor ()
DECLARE SUB InitSprite ()
DECLARE SUB DrawAtCurXY ()
DECLARE SUB DrawSpriteMap ()
DECLARE SUB ClearKbBuffer ()
DECLARE SUB DrawCursor ()
DECLARE SUB DrawBackground ()
'
' Santa's Delivery Rush
' Christmas QB Challenge 2021 Sprite Editor
' Start date: 2021-12-03
' Last Modified: 2022-01-16
' By: Erik Eriksen
'
' Sprite Editor
'
'

TYPE DrawCursor
    x AS INTEGER
    y AS INTEGER
    c AS INTEGER
    drawMode AS INTEGER
    char AS STRING * 1
END TYPE


CONST MAXWIDTH = MAXSPRITEWIDTH
CONST MAXHEIGHT = MAXSPRITEHEIGHT
CONST MINCURSORX = 2
CONST MINCURSORY = 3
CONST MAXCURSORX = MAXWIDTH + MINCURSORX - 1
CONST MAXCURSORY = MAXHEIGHT + MINCURSORY - 1


DIM SHARED sprite(MAXWIDTH, MAXHEIGHT) AS INTEGER

DIM SHARED Cursor AS DrawCursor


CALL InitSprite

quit = FALSE

Cursor.x = MINCURSORX
Cursor.y = MINCURSORY
Cursor.c = 4
Cursor.drawMode = FALSE
Cursor.char = CHR$(178)


spriteFileName$ = "NEWSP001.SDX"

SCREEN 0
WIDTH 80
CLS


CALL DrawBackground


prevIn% = 0

WHILE quit = FALSE


    start! = TIMER

    in% = INP(96)

    LOCATE 18, 40
    PRINT "in%="; INP(96)

    SELECT CASE in%


        'quick keys to change color (ASCII code is shifted by 1)
        CASE 2, 3, 4, 5, 6, 7, 8, 9, 10, 11
            CALL SetQuickCursorColor(in% - 1)


        CASE UPARROW
            Cursor.y = Cursor.y - 1
            IF Cursor.y < MINCURSORY THEN Cursor.y = MINCURSORY

        CASE DOWNARROW
            Cursor.y = Cursor.y + 1
            IF Cursor.y > MAXCURSORY THEN Cursor.y = MAXCURSORY

        CASE RIGHTARROW
            Cursor.x = Cursor.x + 1
            IF Cursor.x > MAXCURSORX THEN Cursor.x = MAXCURSORX


        CASE LEFTARROW
            Cursor.x = Cursor.x - 1
            IF Cursor.x < MINCURSORX THEN Cursor.x = MINCURSORX


        CASE F2KEY
            spriteFileName$ = SaveSprite$(spriteFileName$)


        CASE F3KEY
            spriteFileName$ = LoadSprite$(spriteFileName$)

        CASE FKEY
            CALL FillBackground

        CASE QKEY
            quit = TRUE

        CASE PKEY
            CALL PreviewSprite

        CASE CKEY
            CALL SelectDrawColor

        CASE NKEY
            CALL NewSprite(TRUE)

        CASE RKEY
            CALL ReverseSprite


        CASE SKEY
            CALL DrawAtCurXY

        CASE SPACEKEY

            IF prevIn% <> SPACEKEY THEN

                PLAY "L64G"

                IF Cursor.drawMode THEN
                    Cursor.drawMode = FALSE
                    LOCATE MAXCURSORY + 2, 8
                    COLOR 0, 2
                    PRINT "SELECT"

                ELSE
                    Cursor.drawMode = TRUE
                    LOCATE MAXCURSORY + 2, 8
                    COLOR 0, 2
                    PRINT "DRAW  "
                END IF
            END IF

    END SELECT



    IF Cursor.drawMode THEN
        CALL DrawAtCurXY
    END IF


    CALL DrawSpriteMap

    CALL DrawCursor
    CALL ClearKbBuffer


    LOCATE 18, 50
    PRINT "x,y="; Cursor.x; ","; Cursor.y


    'attept to keep at 20fps
    WHILE TIMER - start! <= .025: WEND

    IF in% <> prevIn% THEN
        prevIn% = in%
    END IF



WEND


CALL EditorExitScreen

END

REM $STATIC
DEFSNG A-Z
SUB DrawAtCurXY

    xTransform% = Cursor.x - MINCURSORX + 1
    yTransform% = Cursor.y - MINCURSORY + 1

    sprite(xTransform%, yTransform%) = Cursor.c



END SUB

DEFINT A-Z
SUB DrawBackground


    COLOR 0, 2
    CLS

    COLOR 0, 7
    PRINT SPACE$(80)
   
    LOCATE 1, 21
    COLOR 15, 7
    PRINT " SANTA'S DELIVERY RUSH - SPRITE EDITOR"
    

    COLOR 0, 2
    LOCATE MAXCURSORY + 1, 2
    PRINT "SPRITE SIZE:"; MAXWIDTH; "x"; MAXHEIGHT


    LOCATE MAXCURSORY + 2, 2
    PRINT "MODE: "
    IF Cursor.drawMode THEN
        LOCATE MAXCURSORY + 2, 8
        PRINT "DRAW  "
    ELSE
        LOCATE MAXCURSORY + 2, 8
        PRINT "SELECT"
    END IF


    LOCATE MAXCURSORY + 3, 2
    PRINT "COLOR:"

    FOR colorVal% = 0 TO 15
        LOCATE MAXCURSORY + 3, colorVal% + 9
        COLOR colorVal%
        PRINT CHR$(219)
    NEXT colorVal%


    LOCATE MAXCURSORY + 4, Cursor.c + 9
    COLOR 0, 2
    PRINT CHR$(24)


    LOCATE 3, 30
    PRINT "Commands:"
    LOCATE 4, 29
    PRINT ""
    LOCATE 5, 25
    PRINT "ARROW KEYS - Move cursor"
    LOCATE 6, 25
    PRINT " SPACE BAR - Toggle draw/select mode"
    LOCATE 7, 25
    PRINT "        F2 - Save sprite"
    LOCATE 8, 25
    PRINT "        F3 - Load sprite"
    LOCATE 9, 25
    PRINT "       0-9 - Quick color select"
    LOCATE 10, 25
    PRINT "         C - Select color"
    LOCATE 11, 25
    PRINT "         F - Fill sprite w/ current color"
    LOCATE 12, 25
    PRINT "         N - New Sprite"
    LOCATE 13, 25
    PRINT "         P - Preview Sprite"
    LOCATE 14, 25
    PRINT "         Q - Quit"
    LOCATE 15, 25
    PRINT "         R - Reverse sprite"
    LOCATE 16, 25
    PRINT "         S - Draw single pixel"
   

    LOCATE 22, 50
    PRINT "Nukem Enterprises"
    LOCATE 23, 50
    PRINT "http://eriks.servehttp.com"

END SUB

SUB DrawCursor

    cursorDrawColor% = Cursor.c

    IF Cursor.char = CHR$(178) THEN
        Cursor.char = CHR$(176)
        IF Cursor.c = sprite(Cursor.x - MINCURSORX, Cursor.y - MINCURSORY) THEN
            cursorDrawColor% = cursorDrawColor% + 1
            IF cursorDrawColor% > 15 THEN
                cursorDrawColor% = 1
            END IF
        END IF

    ELSE
        Cursor.char = CHR$(178)
    END IF


    IF Cursor.x >= MINCURSORX AND Cursor.x <= MAXCURSORX AND Cursor.y >= MINCURSORY AND Cursor.y <= MAXCURSORY THEN

        LOCATE Cursor.y, Cursor.x
        COLOR cursorDrawColor, 0
        PRINT Cursor.char
    ELSE
        LOCATE 15, 15
        COLOR 10, 7
        PRINT "ERROR!:"; Cursor.x; ","; Cursor.y
    END IF

END SUB

DEFSNG A-Z
SUB DrawSpriteMap

    FOR y% = 1 TO MAXHEIGHT
        FOR x% = 1 TO MAXWIDTH

            LOCATE y% + 2, x% + 1
            COLOR sprite(x%, y%)
            PRINT CHR$(219)

        NEXT x%
    NEXT y%

END SUB

DEFINT A-Z
SUB FillBackground

    FOR x% = 1 TO UBOUND(sprite, 1)
        FOR y% = 1 TO UBOUND(sprite, 2)
            sprite(x%, y%) = Cursor.c
        NEXT y%
    NEXT x%


END SUB

DEFSNG A-Z
SUB InitSprite


    FOR y% = 1 TO MAXHEIGHT
        FOR x% = 1 TO MAXWIDTH
            sprite(x%, y%) = 0
        NEXT x%
    NEXT y%



END SUB

FUNCTION LoadSprite$ (existingSpriteFileName$)
  

    COLOR 0, 7
    CLS

    PRINT "LOAD SPRITE DATA"
    PRINT ""
    PRINT

    FILES "*.SDX" 'list current sprite files
    PRINT

    IF LTRIM$(RTRIM$(existingSpriteFileName$)) <> "" THEN
        PRINT "Current file name: ", existingSpriteFileName$
        INPUT "Reload current file (Y/N)"; reloadFile$
        IF UCASE$(reloadFile$) = "Y" THEN
            spriteFileName$ = existingSpriteFileName$
        ELSE
            INPUT "Enter file name to load: ", spriteFileName$
        END IF
    ELSE
        INPUT "Enter file name to load: ", spriteFileName$
    END IF


    IF LTRIM$(RTRIM$(spriteFileName$)) = "" THEN
        LoadSprite$ = existingSpriteFileName$
        CALL ReturnToEditor(5, 15, 15)
        EXIT FUNCTION
    END IF


    INPUT "Load file (Y/N)"; loadFile$
    IF UCASE$(loadFile$) <> "Y" THEN
        LoadSprite$ = existingSpriteFileName$
        CALL ReturnToEditor(5, 15, 15)
        EXIT FUNCTION
    END IF

    IF CheckFileExists%(spriteFileName$) = FALSE THEN
        PRINT "File not found to be loaded..."
        CALL ReturnToEditor(5, 15, 15)
        LoadSprite$ = existingSpriteFileName$
        EXIT FUNCTION
    END IF

   
    CALL NewSprite(FALSE)

    DEF SEG = VARSEG(sprite(1, 1))
    BLOAD spriteFileName$, VARPTR(sprite(1, 1))
    DEF SEG

    LoadSprite$ = spriteFileName$

    PRINT "Loaded sprite: "; spriteFileName$

    CALL ReturnToEditor(5, 25, 15)


END FUNCTION

DEFINT A-Z
SUB NewSprite (interactive%)


    IF interactive% THEN
        LOCATE 10, 5
        COLOR 0, 4
        PRINT SPACE$(30)

        LOCATE 10, 7
        COLOR 15, 4
        PRINT "NEW SPRITE"

        COLOR 0, 7
        LOCATE 11, 5
        PRINT SPACE$(30)
        LOCATE 12, 5
        PRINT SPACE$(30)
        LOCATE 13, 5
        PRINT SPACE$(30)


        LOCATE 12, 7
        COLOR 15, 7
        INPUT "Start new sprite (Y/N)"; createNew$

        IF UCASE$(createNew$) <> "Y" THEN
            CALL DrawBackground
            EXIT SUB
        END IF
    END IF
        

    'zero out array elements.
    FOR x% = 1 TO UBOUND(sprite, 1)
        FOR y% = 1 TO UBOUND(sprite, 2)
            sprite(x%, y%) = 0
        NEXT y%
    NEXT x%

    IF interactive% THEN
        CALL DrawBackground
    END IF

END SUB

DEFSNG A-Z
SUB PreviewSprite

    SCREEN 13
    CLS
    PAINT (1, 1), 7

    COLOR 15
    LOCATE 1, 2
    PRINT "SPRITE PREVIEW"
    LOCATE 3, 2
    PRINT "Normal Size:"

    xOffset% = 30
    yOffset% = 30

    LINE (xOffset%, yOffset%)-(xOffset% + MAXWIDTH + 1, yOffset% + MAXHEIGHT + 1), 15, B


    FOR y% = 1 TO MAXHEIGHT
        FOR x% = 1 TO MAXWIDTH
            PSET (x% + xOffset%, y% + yOffset%), sprite(x%, y%)
        NEXT x%
    NEXT y%


    COLOR 15
    LOCATE 8, 2
    PRINT "Double size:"



    xOffset% = 30
    yOffset% = 70

    LINE (xOffset%, yOffset%)-(xOffset% + (MAXWIDTH * 2) + 2, yOffset% + (MAXHEIGHT * 2) + 2), 15, B


    'This code is GROSS but it works enough for this purpose...
    yIdx% = 1
                                
    FOR y% = 1 TO MAXHEIGHT * 2 + 1
 
        xIdx% = 1

        FOR x% = 1 TO MAXWIDTH * 2 + 1


            PSET (x% + xOffset%, y% + yOffset%), sprite(xIdx%, yIdx%)

            IF x% <> 1 AND x% MOD 2 > 0 THEN
                xIdx% = xIdx% + 1
            END IF


        NEXT x%

    
        IF y% <> 1 AND y% MOD 2 > 0 THEN
            yIdx% = yIdx% + 1
        END IF

    NEXT y%


    CALL ReturnToEditor(2, 15, 15)


END SUB

SUB ReturnToEditor (textX%, textY%, textColor%)


    COLOR textColor%
    LOCATE textY%, textX%
    PRINT "Press ENTER to return to the editor."

    DO: LOOP WHILE INKEY$ <> CHR$(13)

    SCREEN 0
    WIDTH 80
    CALL DrawBackground

END SUB

DEFINT A-Z
SUB ReverseSprite


    DIM lineColors(MAXWIDTH) AS INTEGER

    FOR y% = 1 TO MAXHEIGHT

        FOR x% = MAXWIDTH TO 1 STEP -1
            lineColors(MAXWIDTH - x%) = sprite(x%, y%)
        NEXT x%

        FOR x% = 1 TO MAXWIDTH
            sprite(x%, y%) = lineColors(x%)
        NEXT x%

    NEXT y%


END SUB

DEFSNG A-Z
FUNCTION SaveSprite$ (existingSpriteFileName$)


    SaveSprite$ = existingSpriteFileName$
   
    COLOR 0, 7
    CLS

    PRINT "SAVE SPRITE DATA"
    PRINT ""
    PRINT
   
    IF LTRIM$(RTRIM$(existingSpriteFileName$)) <> "" THEN
        PRINT "Current file name: "; existingSpriteFileName$
        INPUT "Use current file name (Y/N)"; useExisting$

        IF UCASE$(useExisting$) <> "Y" THEN
            INPUT "Enter new file name: ", fileName$
        ELSE
            fileName$ = existingSpriteFileName$
        END IF

    ELSE
        INPUT "Enter file name: ", fileName$
    END IF

    IF RTRIM$(LTRIM$(fileName$)) = "" THEN
        CALL ReturnToEditor(5, 15, 15)
        EXIT FUNCTION
    END IF


    INPUT "Save file (Y/N)"; saveFile$
    IF UCASE$(saveFile$) <> "Y" THEN
        CALL ReturnToEditor(5, 15, 15)
        EXIT FUNCTION
    END IF

    IF INSTR(fileName$, ".") <= 0 THEN
        fileName$ = fileName$ + ".SDX"
    END IF


    PRINT "Saving "; fileName$

    bytes% = LEN(sprite(1, 1))

    PRINT "bytes per sprite pixel: "; bytes%
    PRINT "lbound "; LBOUND(sprite, 1); " : ubound: "; UBOUND(sprite, 1)

    'this works but adds slightly extra overhead.
    'filesize& = (UBOUND(Sprite, 1) + 1) * (UBOUND(Sprite, 2) + 1) * bytes%

    'add addtional 32 bytes to file size (2 array elements)
    filesize& = (UBOUND(sprite, 1)) * (UBOUND(sprite, 2)) * bytes% + 32

    PRINT "file size: "; filesize&

    DEF SEG = VARSEG(sprite(1, 1))
    BSAVE fileName$, VARPTR(sprite(1, 1)), filesize&
    DEF SEG

    lastSaved$ = DATE$ + " " + TIME$

    PRINT "Saved sprite: "; fileName$

  
    SaveSprite$ = fileName$


    CALL ReturnToEditor(5, 20, 15)


END FUNCTION

SUB SelectDrawColor


    printOffsetX% = 9 + Cursor.c
    printOffsetY% = MAXCURSORY + 4

    LOCATE printOffsetY%, printOffsetX%
    COLOR 0, 2
    PRINT CHR$(24)
       

    'highlight label
    COLOR 14, 2
    LOCATE MAXCURSORY + 3, 2
    PRINT "COLOR:"


    'LOCATE 15, 50
    'COLOR 4, 14
    'PRINT "COLOR: "; printOffsetY%; ","; printOffsetX%;


    inColorSelection% = TRUE
    saveColor% = FALSE

    CALL ClearKbBuffer

    WHILE inColorSelection%


        start! = TIMER


        SELECT CASE INP(96)


            CASE LEFTARROW
                LOCATE printOffsetY%, printOffsetX%
                COLOR 2, 2
                PRINT CHR$(24)
                printOffsetX% = printOffsetX% - 1
                IF printOffsetX% <= 9 THEN printOffsetX% = 9
            

            CASE RIGHTARROW
                LOCATE printOffsetY%, printOffsetX%
                COLOR 2, 2
                PRINT CHR$(24)
                printOffsetX% = printOffsetX% + 1
                IF printOffsetX% >= 24 THEN printOffsetX% = 24


            CASE ESCKEY
                inColorSelection% = FALSE


            CASE ENTERKEY
                saveColor% = TRUE
                inColorSelection% = FALSE

        END SELECT


        LOCATE printOffsetY%, printOffsetX%
        COLOR 0, 2
        PRINT CHR$(24)


        WHILE TIMER - start! < .05: WEND

        CALL ClearKbBuffer


    WEND


    'LOCATE printOffsetY%, printOffsetX%
    'COLOR 0, 7
    'PRINT SPACE$(1)

    IF saveColor% THEN
        Cursor.c = printOffsetX% - 9
    ELSE
        LOCATE printOffsetY%, printOffsetX%
        COLOR 0, 2
        PRINT SPACE$(1)

        LOCATE printOffsetY%, Cursor.c + 9
        COLOR 0, 2
        PRINT CHR$(24)
    END IF


    'set label back to original color
    COLOR 0, 2
    LOCATE MAXCURSORY + 3, 2
    PRINT "COLOR:"


END SUB

SUB SetQuickCursorColor (newColor%)

    xDraw% = 9 + Cursor.c
    yDraw% = MAXCURSORY + 4
    LOCATE yDraw%, xDraw%
    COLOR 7, 2
    PRINT SPACE$(1)


    'zero to black instead of light green.
    IF newColor% = 10 THEN newColor% = 0

    Cursor.c = newColor%

    xDraw% = 9 + Cursor.c
    LOCATE yDraw%, xDraw%
    COLOR 0, 2
    PRINT CHR$(24)

END SUB

